import logging

from zope import interface
from zope.component import getUtility
from zope.app.annotation.interfaces import IAnnotations
from zope.interface import implements

from getpaid.core import interfaces, options

from getpaid.pxpay.interfaces import IPXPayStandardOptions, \
     IPXPayWebInterfaceGateway, IPXPayInvalidMessageError, \
     IPXPayPaymentProcessor, IPXPayNetworkError, ICheckoutContext
from getpaid.pxpay import parser
from getpaid.pxpay.exceptions import PXPayException, \
     PXPayInvalidMessageException, PXPayNetworkException

log = logging.getLogger('getpaid.pxpay')

PXPayStandardOptions = options.PersistentOptions.wire(
    "PXPayStandardOptions",
    "getpaid.pxpay",
    IPXPayStandardOptions
    )

class PXPayPaymentAdapter( object ):

    interface.implements( IPXPayPaymentProcessor )

    options_interface = IPXPayStandardOptions

    def __init__( self, context ):
        self.context = context
        self.settings = IPXPayStandardOptions(self.context)
        self.pxpay_gateway = IPXPayWebInterfaceGateway(self.settings)
        self.checkout_context = ICheckoutContext(self.context)

    def _generate_initial_request(self, order):
        return_url = '/'.join((self.checkout_context.root_url,
                              '@@getpaid-order',
                               order.order_id,
                               '@@pxpayprocessresponse'))
        initial_request = parser.InitialRequest()
        initial_request.pxpay_user_id = self.settings.PxPayUserId
        initial_request.pxpay_key = self.settings.PxPayKey
        initial_request.amount_input = self.order_total_price(order)
        initial_request.currency_input = self.settings.PxPaySiteCurrency
        initial_request.merchant_reference = self.settings.MerchantReference
        initial_request.transaction_type = "Purchase"
        initial_request.transaction_id = order.order_id
        initial_request.transaction_data_1 = order.order_id
        initial_request.url_failure = return_url
        initial_request.url_success = return_url

        state_valid, errors = initial_request.state_validate()
        if not state_valid:
            raise PXPayException(errors)
        return initial_request

    def authorize( self, order, payment, request=None):
        initial_request = self._generate_initial_request(order)
        try:
            data = self.pxpay_gateway.send_message(initial_request)
        except PXPayNetworkException:
            getUtility(IPXPayNetworkError)(self.context, request,
                                           order.order_id)
            return

        response_message = parser.InitialResponse(data)
        if not response_message.is_valid_response:
            getUtility(IPXPayInvalidMessageError)(self.context, request,
                                                  order.order_id,
                                                  response_message)
            return
        log.info("Recieved: %s" % response_message)
        payment_url = response_message.request_url
        order.finance_workflow.fireTransition("authorize")
        request.response.redirect(payment_url)

    def order_total_price(self, order):
        """
        pxpay requires a specific format for amounts.
        see : http://www.paymentexpress.com/technical_resources/ecommerce_hosted/pxaccess.html#amountinput
        """
        return '%.2f' % order.getTotalPrice()

    def capture( self, order, amount ):
        return  interfaces.keys.results_async

    def refund( self, order, amount ):
        pass




from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class PXPAYCommunicationError(BrowserView):

    _template = ViewPageTemplateFile('templates/pxpay-communication-error.pt')

    def __call__(self):
        return self._template()



import os

class PXPayException(Exception):
    def __init__(self, err):
        self.err = err

    def __str__(self):
        if type(self.err) is dict:
            return os.linesep.join([' : '.join((x[0],str(x[1]))) \
                                    for x in self.err.items()])
        else:
            return repr(self.err)


class PXPayInvalidMessageException(PXPayException):
    pass

class PXPayNetworkException(PXPayException):
    pass




import logging
from zc import ssl
from zope.component import adapts
from zope.interface import implements

from getpaid.pxpay import parser
from getpaid.pxpay.config import *
from getpaid.pxpay.interfaces import IPXPayWebInterfaceGateway, \
     IPXPayStandardOptions
from getpaid.pxpay.exceptions import PXPayNetworkException

log = logging.getLogger('getpaid.pxpay')

class PXPayWebInterfaceGateway( object ):

    implements(IPXPayWebInterfaceGateway)
    adapts(IPXPayStandardOptions)

    offline_testmode = False
    offline_testdata = {'returnresponse':'payment-accepted',
                        'initialresponse':'valid'}

    def __init__(self, context):
        self.context = context
        self.server_type = context.PxPayServerType

    def set_offline_testmode(self, value, data={}):
        self.offline_testmode = value
        self.offline_testdata.update(data)

    def send_message(self, message, timeout=None):
        """
        Creates the HTTPS/POST connection to the PaymentExpress PXPay
        server sends an xml message and returns the response body -
        which is usually another XML message.
        """
        if self.offline_testmode:
            # hand off to an offline test mode
            return self.send_message_offline_testmode(message)
        server = SERVER_DETAILS.get(self.server_type, {})
        server_name = server.get('host')
        server_path = server.get('path')
        try:
            conn = ssl.HTTPSConnection(server_name, timeout)

            # setup the HEADERS
            conn.putrequest('POST', server_path)
            conn.putheader('Content-Type', 'application/x-www-form-urlencoded')
            conn.putheader('Content-Length', len(message))
            conn.endheaders()

            log.info("About to send: %s" % message)
            conn.send(message.generateXML())
            return conn.getresponse().read()
        except Exception, e:
            errors = {'error':e,
                      'msg': 'communication error with pxpay gateway'}
            raise PXPayNetworkException(errors)

    def send_message_offline_testmode(self, message):

        if isinstance(message, parser.InitialRequest):
            if self.offline_testdata['initialresponse'] == 'valid':
                return initialresponse_test_data_valid
            if self.offline_testdata['initialresponse'] == 'invalid':
                return initialresponse_test_data_invalid
        if isinstance(message, parser.ReturnRequest):
            if self.offline_testdata['returnresponse'] == 'payment-accepted':
                return returnresponse_test_data_payment_accepted
            if self.offline_testdata['returnresponse'] == 'payment-declined':
                return returnresponse_test_data_payment_declined
            if self.offline_testdata['returnresponse'] == 'invalid':
                return returnresponse_test_data_invalid

initialresponse_test_data_valid = """<Request valid="1"><URI>https://www.paymentexpress.com/pxpay/pxpay.aspx?userid=TestAccount&amp;request=e88cd9f2f6f301c712ae2106ab2b6137d86e954d2163d1042f73cce130b2c 88c06daaa226629644dc741b16deb77ca14ce4c59db84929eb0280837b92bd2ffec 2fae0b9173c066dab48a0b6d2c0f1006d4d26a8c75269196cc540451030958d257c1 86f587ad92cfa7472b101ef72e45cda3bf905862c2bf58fc214870292d6646f7c4ad 02a75e42fc64839fc50cea8c17f65c6a9b83b9c124e2f20844b63538e13a8cff17ec d8f165aee525632fd3661b591626f5fb77725ade21648fed94553f43bfa69acf3557 0ff8fdcbaf8a13a3fa7deb244017e41749e652a3549a5dbe20c6c3a7a66aa5901e3f 87150f7fc</URI></Request>"""

initialresponse_test_data_invalid = """<Request valid="0"><URI>https://www.paymentexpress.com/pxpayerr?UserId=testid&amp;Err=0eInvalidKey</URI></Request>"""


returnresponse_test_data_payment_accepted = """<Response valid="1"><Success>1</Success><TxnType>Purchase</TxnType><CurrencyInput>NZD</CurrencyInput><MerchantReference>None</MerchantReference><TxnData1 /><TxnData2 /><TxnData3 /><AuthCode>010026</AuthCode><CardName>Visa</CardName><CardHolderName>TEST</CardHolderName><CardNumber>411111........11</CardNumber><DateExpiry>0909</DateExpiry><ClientInfo>219.89.194.68</ClientInfo><TxnId>123456789</TxnId><EmailAddress /><DpsTxnRef>0000000404cffcc8</DpsTxnRef><BillingId /><DpsBillingId /><AmountSettlement>100.00</AmountSettlement><CurrencySettlement>NZD</CurrencySettlement><TxnMac>BD43E619</TxnMac><ResponseText>APPROVED</ResponseText></Response>"""

returnresponse_test_data_payment_declined = """<Response valid="1"><Success>0</Success><TxnType>Purchase</TxnType><CurrencyInput>NZD</CurrencyInput><MerchantReference>None</MerchantReference><TxnData1 /><TxnData2 /><TxnData3 /><AuthCode /><CardName>Visa</CardName><CardHolderName>HFM TEST</CardHolderName><CardNumber>411111........11</CardNumber><DateExpiry>0908</DateExpiry><ClientInfo>125.238.25.95</ClientInfo><TxnId>234567891</TxnId><EmailAddress /><DpsTxnRef>0000000404c5bee6</DpsTxnRef><BillingId /><DpsBillingId /><AmountSettlement>4.76</AmountSettlement><CurrencySettlement>NZD</CurrencySettlement><TxnMac>BD43E619</TxnMac><ResponseText>DECLINED</ResponseText></Response>"""

returnresponse_test_data_invalid = """<Response valid="0"></Response>"""



from Acquisition import Implicit
class FauxUser( Implicit ):

    def __init__( self, id, name=None, roles={}, groups={} ):

        self._id = id
        self._name = name
        self._roles = roles
        self._groups = groups

    def getId( self ):

        return self._id

    def getUserName( self ):

        return self._name

    def getRoles( self ):

        return self._roles.keys()

    def _addRoles(self, roles):
        for role in roles:
            self._roles[role] = 1

    def getGroups( self ):

        return self._groups.keys()

    def _addGroups(self, groups):
        for group in groups:
            self._groups[group] = 1

    def __repr__( self ):

        return '<FauxUser: %s>' % self._id


from zope.interface import implements
from hurry.workflow import workflow
from hurry.workflow import interfaces as iworkflow

from getpaid.core.workflow.order import create_finance_workflow
from getpaid.core.interfaces import workflow_states
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid')

class FinanceWorkflow( workflow.Workflow ):
    implements( iworkflow.IWorkflow )
    def __init__( self ):
        fs = workflow_states.order.finance
        transitions = create_finance_workflow()
        transitions.append(workflow.Transition(
            transition_id = 'processor-charging-cancelled',
            title = _(u'Processor Cancel from Charging'),
            source = fs.CHARGING,
            destination = fs.CANCELLED_BY_PROCESSOR,
            trigger = iworkflow.SYSTEM, ))
        super( FinanceWorkflow, self).__init__( transitions )


FinanceWorkflowAdapter, _, _ = workflow.ParallelWorkflow(
    workflow.AdaptedWorkflow( FinanceWorkflow() ),
    workflow_states.order.finance.name,
    )
